/*
 * Copyright (c) 2003 Benedikt Meurer <benedikt.meurer@unix-ag.uni-siegen.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <libxfce4mcs/mcs-client.h>
#include <libxfce4util/util.h>
#include <libxfcegui4/dialogs.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>

#include "manager.h"

/*
 * Start a default XFce4 session
 */
static gboolean
start_default_session(void)
{
	GError *error;

	if (!g_spawn_command_line_async("/bin/sh " DEFAULT_SESSION, &error)) {
		xfce_err("The session manager was unable to start the\n"
			 "default session. This is most often caused\n"
			 "by a broken installation of the session manager.\n"
			 "Please contact your local system administrator\n"
			 "and report the problem.");
		g_idle_add((GSourceFunc)exit, GUINT_TO_POINTER(EXIT_FAILURE));
	}

	return(FALSE);
}

/*
 * Run a sanity check before logging in
 */
static gboolean
sanity_check(gchar **message)
{
	gchar *path;

	path = (gchar *)xfce_get_userdir();

	if (!g_file_test(path, G_FILE_TEST_IS_DIR) && mkdir(path, 0755) < 0) {
		*message = g_strdup_printf(
			"Unable to create users XFce settings\n"
			"directory %s: %s", path, g_strerror(errno));
		return(FALSE);
	}

	path = xfce_get_userfile("sessions", NULL);

	if (!g_file_test(path, G_FILE_TEST_IS_DIR) && mkdir(path, 0755) < 0) {
		*message = g_strdup_printf(
			"Unable to create users XFce session\n"
			"directory %s: %s", path, g_strerror(errno));
		return(FALSE);
	}

	g_free(path);

	return(TRUE);
}

int
main(int argc, char **argv)
{
	gchar *message;
	
	gtk_init(&argc, &argv);

	/* run a sanity check before we start the actual session manager */
	if (!sanity_check(&message)) {
		xfce_err("%s", message);
		return(EXIT_FAILURE);
	}

	if (!manager_init())
		g_error("Unable to initialize session manager");

	/* make sure the MCS manager is running */
	if (!mcs_client_check_manager(GDK_DISPLAY(),
				DefaultScreen(GDK_DISPLAY()),
				"xfce-mcs-manager")) {
		xfce_err("The session manager was unable to start the\n"
			 "Multi-Channel settings manager. This is most\n"
			 "often caused by a broken XFce installation.\n"
			 "Please contact your local system administrator\n"
			 "and report the problem.");
		return(EXIT_FAILURE);
	}

	/*
	 * the manager was unable to restart a previous session, so we
	 * simply start a new default session.
	 */
	if (!manager_restart())
		g_idle_add((GSourceFunc)start_default_session, NULL);

	gtk_main();

	if (!manager_save())
		g_warning("Unable to save session");

	return(EXIT_SUCCESS);
}

